/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.ftp;

import java.io.File;
//import java.io.FileOutputStream;
//import java.io.FileInputStream;

import com.nemmco.participantbatcher.process.FileManager;
import com.nemmco.participantbatcher.util.Constants;

/**
 * Provides for communication by a file share instead of FTP.
 * ShareProcesser.java
 *
 * Created on 28 February 2005, 18:12
 * @author Mike Muir
 * version 1.4.0 - 28-Feb-2005 Added share as alternative to FTP
 * version 1.4.4 - 24-Apr-2006 Throw some exceptions as required by a ComProcess
 * version 1.5.0 - 08-Aug-2006 Added webDAV
 * version 1.5.4 - 17-Aug-2007 Changed rename call to fix logging
 * version 2.0.1 - 15-Nov-2010 Peer review
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 *
 */
public class ShareProcesser implements ComProcess {

   private static FileManager fm = new FileManager();

   /** Creates a new instance of ShareProcesser */
   public ShareProcesser() {
      //fm = new FileManager();
   }

   /** Abort the operation in progress - does nothing */
   public boolean abort() throws ComException {
      return true;
   }

   /** Change working directory - does nothin g as all calls have path included */
   public boolean changeWorkingDirectory(String pathName) throws ComException {
      return true;
   }

   /** Connect - as the share is a permanaent connection just prove the share by doing a dir command */
   public void connect(String serverName) throws ComException {
      if (fm.scanDir(Constants.getConstant().getPNInboxDir(), null) == null) { // v1.4.4
         throw new ComException("" + Constants.getConstant().getPNInboxDir() + " not shared", null);
      }
   }

   /**
    * Set the port working range in active ftp mode.  Dosent in ShareProcesser.
    * <p>
    * @param lowLimit int
    * @param highLimit int
    * @exception ComException If an error occurs while setting range
    */
   public void setActivePortRange(int lowLimit, int highLimit) throws ComException {}

   /**
    * set the domain for webDAV
    * @param domain String
    */
   public void setDomain(String domain) throws ComException {
      // do nothing for Share
   }

   /** delete the file specified
    * @param pathname - specifies the file to delete */
   public boolean deleteFile(String pathname) throws ComException {
      return fm.deleteFile(pathname);
   }

   /* Disconnect from share - does nothing */
   public void disconnect() throws ComException {}

   /** Checks if connected - as the share is permanent do a dir to prove share
    * @return boolean true on success */
   public boolean isConnected() throws ComException {
      if (fm.scanDir(Constants.getConstant().getPNInboxDir(), null) == null) { //v1.4.4
         throw new ComException("" + Constants.getConstant().getPNInboxDir() + " not shared", null);
      }
      return true;
   }

   /** Checks if share working - as the share is permanent do a dir to prove share
    * @return boolean true on success */
   public boolean isPositiveReply() throws ComException {
      if (fm.scanDir(Constants.getConstant().getPNInboxDir(), null) == null) { // v1.4.4
         throw new ComException("" + Constants.getConstant().getPNInboxDir() + " not shared", null);
      }
      return true;
   }

   /** provide a file list
    * @param pathName - the path to do dir of
    * @return String[] list of filenames */
   public String[] listNames(String pathName) throws ComException {
      return fm.scanDir(pathName, null);
   }

   /** Login - provided to satisfy interface - does nothing
    * @return true */
   public boolean login(String userName, String password) throws ComException {
      return true;
   }

   /** Logout - provided to satisfy interface - does nothing
    * @return true */
   public boolean logout() throws ComException {
      return true;
   }

   /** Rename a file
    * @param from - old file name
    * @param to - new file name
    * @return boolean success */
   public boolean rename(String from, String to) throws ComException {
      return fm.renameFile(from, to);
   }

   /** Get a file
    * @param remote - filename and path of the file to get
    * @param local - File to create it on
    * @return boolean - success */
   public boolean retrieveFile(String remote, java.io.File local) throws ComException {
      File remoteFile = new File(remote);
      boolean statusReturn;
      try {
         fm.copyFile(remoteFile, local);
         statusReturn = true;
      } catch (Exception e) {
         statusReturn = false;
      }
      return statusReturn;
   }

   /** setDataTimeout - provided to satisfy interface - does nothing
    * @return true */
   public void setDataTimeout(int timeout) {}

   /** setFileType - provided to satisfy interface - does nothing
    * @return true */
   public boolean setFileType(String fileType) throws ComException {
      return true;
   }

   /** Put a file
    * @param remote - filename and path of the file to put
    * @param local - File to put
    * @return boolean - success */
   public boolean storeFile(String remote, java.io.File local) throws ComException {
      File remoteFile = new File(remote);
      boolean statusReturn;
      try {
         fm.copyFile(local, remoteFile);
         statusReturn = true;
      } catch (Exception e) {
         statusReturn = false;
      }
      return statusReturn;
   }

}
