/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2006
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */

package com.nemmco.participantbatcher.process;

/**
 * Extends Thread to add some common functions used in participant batcher associated
 * with stopping and sleeping
 *
 * @author mikem
 * version 1.5.1 - 5-Nov-2006 Creation to improve reponsiveness when stopping threads
 * @version 2.0.1 - 15-Nov-2010 Peer review work
 */
public class PbThread extends Thread {

   private volatile boolean stop = false;

   /** Creates a new instance of PbThread */
   public PbThread() {}

   /** allow thread a smooth stop by setting a flag
    * also sets flags in send and recieve process
    */
   public void requestStop() {
      stop = true;
   }

   /**
    * Get the status of the stop variable
    *
    * @return boolean true if the thread has been requested to stop
    */
   public boolean getStop() {
      return stop;
   }

   /**
    * This will wait a time in millisec but will respond quicker
    * if a requestStop has been executed
    *
    * @param millis long the time to wait in milliseconds
    */
   public void partSleep(long millis) {
      long waitCounter = millis;
      long waitOnce = 0;
      do {
         if (waitCounter > 1000) {
            waitOnce = 1000;
         } else {
            waitOnce = waitCounter;
         }
         waitCounter = waitCounter - waitOnce;
         try {
            Thread.sleep(waitOnce); // java 1.6
         } catch (Exception e) {
            ;
         }
      } while (!stop && (waitCounter > 0));
   }

}
