/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 * 28-Aug-2012     Oleg Pevzner       7984 Java upgrade from 6 to 7
 */
package com.nemmco.participantbatcher.util;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

import java.text.DateFormat;

import java.io.Serializable;

/**
 * This class stores the running history data of the Participant Batcher.
 * It has the ability to load from a serialized version of the data
 * from disk if the file exists.
 *
 * version 1.4.2 Created on 12 January 2006, 18:44
 * version 1.4.4 6-May-2006 Fixed javaDoc and coding standards
 * version 2.0.1 - 15-Nov-2010 Peer review work
 * @version 2.0.2 - 19-Nov-2010 Peer review work
 *
 * @author mikem
 */
public class HttpDataStore implements Serializable {

   private static HttpDataStore hds;
   private String[] pniFileList;
   private String[] pnoFileList;
   private FileCounter zipsSent; // sending
   private int acksRecieved;
   private FileCounter zipsRecieved; // recieving
   private int acksSent;
   private boolean connected;
   private DateFormat dateFormat;
   private Date startDate;
   private Date updateDate;
   private int pliZipFileCount;
   private int pliAckFileCount;
   private int ploZipFileCount;
   private int ploAckFileCount;
   private static final long serialVersionUID = 100100142001001L;

   /** Creates a new instance of HttpDataStore - singleton */
   private HttpDataStore() {

      startDate = new Date();
      updateDate = new Date();
      dateFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.LONG);
      zipsSent = new FileCounter();
      zipsRecieved = new FileCounter();
      acksRecieved = 0;
      acksSent = 0;
      connected = false;
      pliZipFileCount = 0;
      ploZipFileCount = 0;
      pliAckFileCount = 0;
      ploAckFileCount = 0; //not used

   }

   /** allows the user to get this instance.  This class is a singleton
    * so you must get a reference.
    */
   public static HttpDataStore getHttpDataStore() {
      if (hds == null) {
         HttpShutdownHook hsh = new HttpShutdownHook();
         HttpDataStore tempHds = hsh.loadFile();
         if (tempHds == null) {
            tempHds = new HttpDataStore();
         } else {
            // fixup the data as needed from the reload
            tempHds.setStartDate(new Date());
            tempHds.setConnected(false);
         }
         hds = tempHds;
      }
      return hds;
   }

   /** unit test main to see if it works. */
   public static void main(String[] args) {
      HttpDataStore hdds;
      hdds = HttpDataStore.getHttpDataStore();

      for (;;) {
         try {
            Thread.sleep(10100);
         } catch (Exception e) {
            ;
         }
         hdds.zipsSent.setFiles(10);
         int crap = hdds.getZipsSentLastHour();
            System.out.println("Now " + hdds.getUpdateDateTime() +
                    " Started " + hdds.getStartDateTime() +
                    " Files per hour " + crap +
                    " i=0 " + hdds.getZipsSentPerHour(0) +
                    " i=1 " + hdds.getZipsSentPerHour(1));
      }
   }

   /** Set the Participant NEMMCO inbox list
    *
    * @param pniFileList String[]
    */
   public void setPniFileList(String[] pniFileList) {
      updateDate = new Date();
      // copy to avoid indirect effects
      if (pniFileList != null) {
         this.pniFileList = new String[pniFileList.length];
         for (int i = 0; i < pniFileList.length; i++) {
            this.pniFileList[i] = pniFileList[i];
         }
      }
   }

   /** Set the Participant NEMMCO outbox list
    *
    * @param pnoFileList String[]
    */
   public void setPnoFileList(String[] pnoFileList) {
      updateDate = new Date();
      if (pnoFileList != null) {
         this.pnoFileList = new String[pnoFileList.length];
         for (int i = 0; i < pnoFileList.length; i++) {
            this.pnoFileList[i] = pnoFileList[i];
         }
      }
   }
   /* Set the count of zip files sent.
     *
     * @param zipsSent int
     */

   public void setZipsSent(int files) {
      updateDate = new Date();
      this.zipsSent.setFiles(files);
   }

   /** Set the count of zip files received.
    *
    * @param files zipsRecieved
    */
   public void setZipsRecieved(int files) {
      updateDate = new Date();
      this.zipsRecieved.setFiles(files);
   }

   /** Set the count of acks sent to NEMMCO.
    *
    * @param files acksSent int
    */
   public void setAcksSent(int files) {
      updateDate = new Date();
      this.acksSent = files;
   }

   /** Set count of acks recieved from NEMMCO.
    *
    * @param files acks recieved int
    */
   public void setAcksRecieved(int files) {
      updateDate = new Date();
      this.acksRecieved = files;
   }

   /** Set the link connected status.
    *
    * @param connected boolean true =connected
    */
   public void setConnected(boolean connected) {
      updateDate = new Date();
      this.connected = connected;
   }

   /** Set the count of zip files in the P Local Inbox
    *
    * @param pliZipFileCount int
    */
   public void setPliZipFileCount(int pliZipFileCount) {
      updateDate = new Date();
      this.pliZipFileCount = pliZipFileCount;
   }

   /** Set the count of zip files in the local outbox.
    *
    * @param ploZipFileCount int
    */
   public void setPloZipFileCount(int ploZipFileCount) {
      updateDate = new Date();
      this.ploZipFileCount = ploZipFileCount;
   }

   /** Set the count of ack files in the local inbox.
    *
    * @param pliAckFileCount int
    */
   public void setPliAckFileCount(int pliAckFileCount) {
      updateDate = new Date();
      this.pliAckFileCount = pliAckFileCount;
   }

   /** Set the count of local ack files.
    *
    * @param ploAckFileCount int
    */
   public void setPloAckFileCount(int ploAckFileCount) {
      updateDate = new Date();
      this.ploAckFileCount = ploAckFileCount;
   }

   /** Set the application start date.
    *
    * @param startDate date
    */
   private void setStartDate(Date startDate) {
      this.startDate = startDate;
   }

   // getters
   public String[] getPniFileList() {
      if (pniFileList != null) {
         Arrays.sort(pniFileList);
         return pniFileList.clone();
      } else {
         String[] s = null;
         return s;
      }
   }

   public String[] getPnoFileList() {
      if (pnoFileList != null) {
         Arrays.sort(pnoFileList);
         return pnoFileList.clone();
      } else {
         String[] s = null;
         return s;
      }
   }

   public int getZipsSentLastHour() {
      return zipsSent.getFilesLastHour();
   }

   public int getZipsRecievedLastHour() {
      return zipsRecieved.getFilesLastHour();
   }

   public int getZipsSentPerHour(int index) {
      return zipsSent.getFilesPerHour(index);
   }

   public int getZipsRecievedPerHour(int index) {
      return zipsRecieved.getFilesPerHour(index);
   }

   public int getZipsSent() {
      return this.zipsSent.getCurrentFiles();
   }

   public int getZipsRecieved() {
      return this.zipsRecieved.getCurrentFiles();
   }

   public int getAcksRecieved() {
      return this.acksRecieved;
   }

   public int getAcksSent() {
      return this.acksSent;
   }

   public String getStartDateTime() {
      return dateFormat.format(this.startDate);
   }

   public String getUpdateDateTime() {
      return dateFormat.format(this.updateDate);
   }

   public boolean getConnected() {
      return this.connected;
   }

   public int getPliZipFileCount() {
      return this.pliZipFileCount;
   }

   public int getPloZipFileCount() {
      return this.ploZipFileCount;
   }

   public int getPliAckFileCount() {
      return this.pliAckFileCount;
   }

   public int getPloAckFileCount() {
      return this.ploAckFileCount;
   }

   // internal class for calculating files per hour

   private static class FileCounter implements Serializable {

      private long lastTime;
      private long thisTime;
      private int[] minuteValues;
      private int[] hourValues;
      private int sumHour;
      private volatile int minutePointer;
      private volatile int hourPointer;
      private volatile int filesLastHour;
      private long periodInMilliSec;
      //private int length;
      private static final long serialVersionUID = 100100142001001L;

      // creator initialise
      private FileCounter() {
         periodInMilliSec = 60000;
         lastTime = Calendar.getInstance().getTimeInMillis();
         minutePointer = 0;
         hourPointer = -1;
         filesLastHour = 0;
         minuteValues = new int[60];
         for (int i = 0; i < minuteValues.length; i++) {
            minuteValues[i] = 0;
         }
         hourValues = new int[24];
         for (int i = 0; i < hourValues.length; i++) {
            hourValues[i] = 0;
         }
         sumHour = 0;
      }

      // periodically call with the number of files processed
      private void setFiles(int files) {

         //save data and get the time (always put data into the latest slot)
         thisTime = Calendar.getInstance().getTimeInMillis();

         // have we run for a minute? if so update average
         if ((thisTime - lastTime) > periodInMilliSec) {

            // sum up the files in the last hour
            filesLastHour = 0;
            for (int i = 0; i < minuteValues.length; i++) {
               filesLastHour = filesLastHour + minuteValues[i];
            }

            // reset counters - need to call the routine at least once per hour
            int offset = (int)Math.floor(((double)thisTime - (double)lastTime) / ((double)periodInMilliSec));
            minutePointer = minutePointer + offset;
            if (minutePointer >= minuteValues.length) {

               // may be greter than an hour
               int hourOffset = 0;
               while (minutePointer >= minuteValues.length) {
                  minutePointer = minutePointer - minuteValues.length;
                  hourOffset = hourOffset + 1;
               }

               // now store data to the hour array
               hourPointer = hourPointer + hourOffset;
               if (hourPointer >= hourValues.length) {

                  // may be greater than a day
                  while (hourPointer >= hourValues.length) {
                     hourPointer = hourPointer - hourValues.length;
                  }
                  for (int i = 0; i < hourOffset; i++) {
                     int p = hourPointer - i;
                     while (p < 0) {
                        p = p + hourValues.length;
                     }
                     hourValues[p] = 0;
                  }
               }
               hourValues[hourPointer] = sumHour;
               sumHour = 0;
            }

            // move the time along to the whole value and zero the next counter (and the ones between)
            lastTime = lastTime + periodInMilliSec * (long)offset;
            for (int i = 0; i < offset; i++) {
               int p = minutePointer - i;
               while (p < 0) {
                  p = p + minuteValues.length;
               }
               minuteValues[p] = 0;
            }
         }
         minuteValues[minutePointer] = minuteValues[minutePointer] + files;
         sumHour = sumHour + files;
      }

      // getters
      private int getFilesLastHour() {
         return this.filesLastHour;
      }

      private int getCurrentFiles() {
         return this.minuteValues[minutePointer];
      }

      private int getFilesPerHour(int index) {

         // index of 0 is previous hour last stored
         int workHourPointer = hourPointer - index;
         if (workHourPointer < 0) {
            workHourPointer = workHourPointer + hourValues.length;
         }
         if (workHourPointer < 0) {
            workHourPointer = 0;
         }
         if (workHourPointer >= hourValues.length) {
            workHourPointer = hourValues.length - 1;
         }
         return this.hourValues[workHourPointer];
      }
   }
}
