/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited. 2019
 *
 * 06-Mar-2019     Deepak Gupta       DFCT0022009
 */

package com.nemmco.participantbatcher.util;

import java.util.StringTokenizer;
import java.util.Vector;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MSATSDOMHelper {
	public MSATSDOMHelper() {
		super();
	}

	public static Vector<Element> getElementNodes(Document document, String tagName) {
		Vector<Element> returnElements = null;
		Element element = null;
		String token;
		StringTokenizer st = new StringTokenizer(tagName, "/");

		if (document != null && st.hasMoreTokens()) {
			token = st.nextToken();
			element = document.getDocumentElement();

			if (element != null && element.getNodeName().equals(token)) {
				if (!st.hasMoreTokens()) {
					returnElements = new Vector<Element>();
					returnElements.add(element);
				} else {
					returnElements = getElementNodes(element, st);
				}
			}
		}
		return returnElements;
	}

	public static Vector<Element> getElementNodes(Element element, String tagName) {
		Vector<Element> returnElements = null;
		StringTokenizer st = new StringTokenizer(tagName, "/");

		if (element != null && st.hasMoreTokens()) {
			returnElements = getElementNodes(element, st);
		}

		return returnElements;
	}

	private static Vector<Element> getElementNodes(Element element, StringTokenizer st) {
		NodeList nodeList;
		Node node;
		Vector<Element> ret = new Vector<Element>();

		String token;

		if (element != null && st.hasMoreTokens()) {
			token = st.nextToken();
			nodeList = element.getChildNodes();

			if (nodeList != null) {
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);
					if (node.getNodeType() == Node.ELEMENT_NODE && (node.getNodeName().equals("*") || node.getNodeName().equals(token))) {
						if (!st.hasMoreTokens())
							ret.add((Element)node);
						else
							return getElementNodes((Element)node, st);
					}
				}
			}
		}

		return ret;
	}

	public static Element getFirstChildElementNode(Element element) {
		NodeList nodeList;
		Node node;

		if (element != null) {
			nodeList = element.getChildNodes();

			if (nodeList != null) {
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);
					if (node.getNodeType() == Node.ELEMENT_NODE)
						return (Element)node;
				}
			}
		}

		return null;
	}

	public static Element getElementNode(Document document, String tagName) {
		Element element = null;

		StringTokenizer st;
		String token;
		st = new StringTokenizer(tagName, "/");

		if (document != null && st.hasMoreTokens()) {
			token = st.nextToken();
			element = document.getDocumentElement();

			if (element != null && element.getNodeName().equals(token)) {
				if (!st.hasMoreTokens())
					return element;
				else
					return getElementNode(element, st);
			}
		}

		return null;
	}

	private static Element getElementNode(Element element, StringTokenizer st) {
		NodeList nodeList;
		Node node;

		String token;

		if (element != null && st.hasMoreTokens()) {
			token = st.nextToken();
			nodeList = element.getChildNodes();

			if (nodeList != null) {
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);
					if (node.getNodeType() == Node.ELEMENT_NODE && node.getNodeName().equals(token)) {
						if (!st.hasMoreTokens())
							return (Element)node;
						else
							return getElementNode((Element)node, st);
					}
				}
			}
		}

		return null;
	}

	// This method is used to get value of an element by name
	public static String getElement(Document document, String tagName) {

		// get elements that match
		NodeList elements = document.getElementsByTagName(tagName);

		// is there anything to do?
		if (elements == null) {
			return ""; // no this tag.
		}

		StringBuffer buffer = new StringBuffer();
		int elementCount = elements.getLength();
		for (int i = 0; i < elementCount; i++) {
			Node element = elements.item(i);
			if (element != null) {
				if (element.getFirstChild() != null) {
					buffer.append(element.getFirstChild().getNodeValue()); // v1.4.3
				}
			}
		}

		return buffer.toString();
	}

	// This method is used to get value of an attribute by name
	public static String getAttribute(Document document, String tagName, String attName) {

		String retStr = "";

		// get elements that match
		NodeList elements = document.getElementsByTagName(tagName);

		// is there anything to do?
		if (elements == null) {
			return retStr; // no this tag.
		}

		int elementCount = elements.getLength();
		for (int i = 0; i < elementCount; i++) {
			Element element = (Element)elements.item(i);
			NamedNodeMap attributes = element.getAttributes();
			if (attributes.getNamedItem(attName) != null) {

				int attributeCount = attributes.getLength();
				for (int j = 0; j < attributeCount; j++) {
					Attr attribute = (Attr)attributes.item(j);
					if (attribute.getNodeName().equals(attName)) {
						retStr = attribute.getNodeValue();
					}
				}
			}
		}

		return retStr;
	}

}
