/*
 * The Copyright of this work is vested in the National Electricity Market
 * Management Company Limited and the software is issued in confidence
 * for the purpose only for which it is supplied. It must not be
 * reproduced in whole or in part or used for tendering or manufacturing
 * purposes except under an agreement or with the consent in writing of
 * the National Electricity Market Management Company Limited and then
 * only on the condition that this notice is included in any such
 * reproduction.  No information as to the contents or subject matter of
 * this software or any part thereof arising directly or indirectly there
 * from shall be given orally or in writing or communicated in any manner
 * whatsoever to any third party being an individual firm or company or
 * any employee thereof without the prior consent in writing of the National
 * Electricity Market Management Company Limited.
 *
 * Copyright the National Electricity Market Management Company Limited.  2003
 * Mike Muir
 */

package com.nemmco.participantbatcher.util;

import org.apache.log4j.Logger;

/**
 * This class is used to provide a common interface to the log4j framework
 *
 * @author Mike Muir
 * version 1.3 - 19-May-2004 Creation of log4j logging
 * @version 1.4.4 - 24-Apr-2006 Donet need a logger itself (passed in), changed warning without exception
 */

public class Util_Logger {

   //private static Logger xlogger = Logger.getLogger(Util_Logger.class); v1.4.4

   // =================================================================
   //
   // logging interfaces to the log4j framework
   //

   /** Log a debug method .
    *
    * @param logger the ligger instance
    * @param theMessage string the message
    */
   public static void debug(Logger logger, String theMessage) {
      logger.debug(theMessage);
   }

   /** Log an informational message.  Limit these to essential only.
    *
    * @param logger the ligger instance
    * @param theMessage string the log message
    */
   public static void info(Logger logger, String theMessage) {
      logger.info(theMessage);
   }

   /** Log a warning message.  This indicates the system is not
    *  working as expecetd.
    *
    * @param logger the ligger instance
    * @param theMessage string the log message
    */
   public static void warning(Logger logger, String theMessage) throws ApplicationException {
      Util_Logger.warning(logger, theMessage, null);
   }

   /** Log a warning message.  This indicates the system is not
    *  working as expecetd and an exception exists.
    *
    * @param logger the logger instance
    * @param theMessage string the log message
    * @param e Exception
    */
   public static void warning(Logger logger, String theMessage, Exception e) {
      if (e == null) {
         logger.warn(theMessage); // v1.4.4
      } else if (e instanceof ApplicationException) {} else {
         logger.warn(theMessage, e);
      }
   }

   /** Log an error message.  This indicates the system is not
    *  working as expecetd and should just stop for the user to fix it.
    *
    * @param logger the ligger instance
    * @param theMessage string the log message
    */
   public static void error(Logger logger, String theMessage) throws ApplicationException {
      Util_Logger.error(logger, theMessage, null);
   }

   /** Log an error message.  This indicates the system is not
    * working as expecetd and an exception exists and the system
    * will stop for the user to fix it.
    *
    * @param logger the logger instance
    * @param theMessage string the log message
    * @param e Exception
    */
   public static void error(Logger logger, String theMessage, Exception e) throws ApplicationException {
      if (e == null) {
         logger.error(theMessage);
         throw new ApplicationException("", theMessage, "ERROR", "");
      } else if (e instanceof ApplicationException) {
         throw (ApplicationException)e;
      } else {
         logger.error(theMessage, e);
         throw new ApplicationException("", theMessage, "ERROR", "");
      }
   }

}
